**THIS CODE WAS WRITTEN FOR CBE 782 BY J.REED (3/2011)
**Due to GAMS demo limits not all reactions are included for distinguishing between alternate optima
$offlisting

$onecho > cplex.opt
eprhs 1e-9
epopt 1e-9
epint 1e-9
$offecho

*Read in the appropriate S matrix
$include EcoliCoreTextbookModel.gms

*Place limits on the exchange fluxes based on the minimal media
*for a negative flux through the exchange reactions implies that
*the metabolites are being taken up or consumed by the cell.
*By default the upperlimits of the exchange fluxes are all set to
*the Vmax, indicating that the cell can secrete any of the extracellular
*metabolites
UpperLimits(j)=Vmax;
*CARBON SOURCE: select upper and lower limits for exchange flux
LowerLimits('EX_glc_e')=-5;
UpperLimits('EX_glc_e')=0;
*allow co2,pi,o2,h,h2o to be taken up by the cell
LowerLimits('EX_co2_e')=-Vmax;
LowerLimits('EX_h2o_e')=-Vmax;
LowerLimits('EX_h_e')=-Vmax;
LowerLimits('EX_o2_e')=0;
LowerLimits('EX_pi_e')=-Vmax;


*Define reactions that are used in distinguishing between alternate optimal solutions
*if you want all reactions to be used just use "alias(j,subj)" without defining the set subj(j)
Sets subj(j) /ACKr,ACONT,ADHEr,ADK1,AKGDH,ATPS4r,CS,CYTBD,ENO,FBA,FBP,FRD,FUM,G6PDH2r,GAPD,GND,
ICDHyr,ICL,LDH_D,MALS,MDH,ME1,ME2,NADH11,NADTRHD,PDH,PFK,PFL,PGI,PGK,PGL,PGM,PPC,PPCK,PPS,PTAr,
PYK,RPE,RPI,SUCD1i,SUCD4,SUCOAS,TALA,THD2,TKT1,TKT2,TPI/

Set k how many alternate solutions to look for /alternate1*alternate20/;

Parameter 
c(j) selects which fluxes are maximized in FBA /Biomass 1/
Objcrit stores the optimal value for the FBA objective function 
PreviousNZ(subj,k) fluxes that are non-zero in previous solutions
EquivOptima(j,k) saves the flux distributions from previous solutions
PreviousSum(k) stores how many non-zero fluxes there were in previous solutions
epsilon /0.000000001/;

*Initialize PreviousSum and PreviousNZ so that future iterations don't constrain
*the current iteration (i.e. PreviousSum is a large number and PreviousNZ is zero)
PreviousSum(k)=card(j)+1;
PreviousNZ(subj,k)=0;

Variables
v(j) flux values through reaction in network
Obj  this is the value of the objective function for the FBA solutions
YObj the number of non-zero fluxes;

v.lo(j)=LowerLimits(j);
v.up(j)=UpperLimits(j);

Binary variable y(subj);

Equations
massbalance(i) mass balance equations for each metabolite
calcobj calculates the dot product of the c vector the flux vector
integercut(k) ensures that at least one non-zero flux from previous iterations is zero in the current solution
upperbound(subj) constrains fluxes by integer variable y
lowerbound(subj) constrains fluxes by integer variable y
sumy calculates the number of fluxes that are not constrained to zero;

massbalance(i).. sum( j,S(i,j)*v(j) )=e=0;
calcobj.. Obj=e=sum( j,c(j)*v(j) );
integercut(k).. sum(subj, y(subj)*PreviousNZ(subj,k))=l=PreviousSum(k)-1;
upperbound(subj).. v(subj)=l=( y(subj)*UpperLimits(subj) );
lowerbound(subj).. v(subj)=g=( y(subj)*LowerLimits(subj) );
sumy.. YObj=e=sum(subj,y(subj));

model FBA /massbalance, calcobj/;
model AltOptima /massbalance,calcobj,integercut,lowerbound,upperbound,sumy/;
FBA.optfile=1; AltOptima.optfile=1; AltOptima.OptCr=0;

*Use FBA and AltOptima to find first optimal solution using the smallest number of reactions
solve FBA using lp maximizing Obj;
Obj.fx=Obj.l;
solve AltOptima using mip minimizing YObj;
EquivOptima(j,'alternate1')=v.l(j);

alias (k,temp);

*Use AltOptima to find other equivalent solutions
loop(temp,
  if(ord(temp)>1,
    if(AltOptima.modelstat=1,
         PreviousSum(temp-1)=0;
         loop(subj, if( abs(v.l(subj))>epsilon, PreviousNZ(subj,temp-1)=1; PreviousSum(temp-1)= PreviousSum(temp-1)+1; );  );
         solve AltOptima using mip minimizing YObj;
         EquivOptima(j,temp)=v.l(j););  );  
   );


*This section writes out the results to a txt file, only prints the alternate optimal solutions
file result /AltSolutions_CoreModel_results.txt/;
put result;
result.pw=500; result.pc=5; result.ps=130;
loop(j,
   put j.tl;
   loop(k, if(PreviousSum(k)<card(j)+1, put EquivOptima(j,k):10:6;); );
   put /;);
putclose result;