**THIS CODE WAS WRITTEN BY J.KIM AND MODIFIED FOR CBE782 BY J.REED (04/2011)

$onecho > cplex.opt
eprhs 1e-8
epopt 1e-8
epint 1e-9
$offecho

$offlisting
$onecho > cplex.opt
eprhs 1e-9
epopt 1e-9
epint 1e-9
$offecho

*Read in the appropriate S matrix
$include EcoliCoreTextbookModel.gms
S(i,'Biomass')=1.3*S(i,'Biomass');
LowerLimits('ATPM')=7.6;

UpperLimits(j)=Vmax;
*CARBON SOURCE: select upper and lower limits for exchange flux
LowerLimits('EX_glc_e')=-18.5;
*allow co2,pi,o2,h,h2o to be taken up by the cell
LowerLimits('EX_co2_e')=-Vmax;
LowerLimits('EX_h2o_e')=-Vmax;
LowerLimits('EX_h_e')=-Vmax;
LowerLimits('EX_o2_e')=0;
LowerLimits('EX_pi_e')=-Vmax;

* Target growth rate
LowerLimits('Biomass')=0.01;

UpperLimits('EX_glc_e')=LowerLimits('EX_glc_e');

Sets
store how many optknock solutions you want to find /solution1*solution3/
exclude(j) subset of reactions that you do not want OptKnock to consider (usually transporters and exchange fluxes)
/Biomass,EX_ac_e,EX_akg_e,EX_co2_e,EX_etoh_e,EX_for_e,EX_fum_e,EX_glc_e,EX_h2o_e,EX_h_e,EX_lac_D_e,EX_o2_e,EX_pi_e,EX_pyr_e,EX_succ_e
ACt2r,AKGt2r,CO2t,D_LACt2,ETOHt2r,FORt,FUMt2_2,GLCpts,H2Ot,O2t,PIt,PYRt2r,SUCCt2_2,SUCCt2b/;

Parameter
outer_obj(j) used to define the objective function for Optknock /EX_ac_e 1/
inner_obj(j) used to define the objective function for inner problem /Biomass 1/
Umax used to limit dual variables /1000/
maxKnockouts maximum number of knockouts /1/
storeA(j,store) stores the a values from previous iterations
storeRxns(store) stores the number of reactions that were removed from previous iterations
storeV(j,store) stores the fluxes at the maximum growth rate from previous iterations
alpha /1e-4/;

Variables
v(j) flux values through reaction in network
u1(i) dual variable for S*v = 0
u2(j) dual variable for   v = 0
Obj  this is the value of the objective function for the FBA solutions;

Positive Variables
u3(j) dual variable for -v =l= -vmin
u4(j) dual variable for  v =l=  vmax;

Binary Variable
a(j);
a.fx(exclude)=1;

Equations
massbalance(i) mass balance equations for each metabolite
dualbalance(j)
primaldual
vupper(j)
vlower(j)
u2upper(j)
u2lower(j)
u3upper(j)
u4upper(j)
maxrxn
intcut
calcobj calculates the dot product of the c vector the flux vector;

massbalance(i).. sum( j,S(i,j)*v(j) )=e=0;
dualbalance(j).. sum( i,S(i,j)*u1(i) )+u2(j)-u3(j)+u4(j)=e=inner_obj(j);
primaldual.. sum( j,inner_obj(j)*v(j) )=e= -sum( j,LowerLimits(j)*u3(j) )+sum( j,UpperLimits(j)*u4(j));
vupper(j).. v(j)=l=a(j)*UpperLimits(j);
vlower(j).. v(j)=g=a(j)*LowerLimits(j);
u2lower(j).. u2(j)=g=-(1-a(j))*Umax;
u2upper(j).. u2(j)=l=(1-a(j))*Umax;
u3upper(j).. u3(j)=l=a(j)*Umax;
u4upper(j).. u4(j)=l=a(j)*Umax;
maxrxn.. sum( j,1-a(j) )=l=maxKnockouts;
intcut(store).. sum( j,storeA(j,store)*a(j))+1=l=card(j)-storeRxns(store);
calcobj.. Obj=e=sum( j,outer_obj(j)*v(j) )-alpha*sum( j,1-a(j) );

Model FBA /massbalance, vupper, vlower, calcobj/;
Model Optknock /massbalance, dualbalance, vupper, vlower, u2lower, u2upper, u3upper, u4upper, maxrxn, primaldual, intcut, calcobj/;
Optknock.OptFile=1;
Optknock.reslim=1000;
Optknock.iterlim=50000;
Optknock.optcr=0;

storeA(j,store)=0;
storeRxns(store)=0;
alias(temp,store);

loop(temp,
            solve Optknock using mip maximizing Obj;
            storeA(j,temp)=a.l(j);
            storeRxns(temp)=sum( j,1-a.l(j) );
            storeV(j,temp)=v.l(j);
);

*This section writes out the results to a txt file
file result /OptKnock_Core_Results.txt/;
result.pw=500; result.pc=5; result.ps=130;
put result;
loop(store$((storeRxns(store)>0) and (storeRxns(store)<>50)),
put 'Number of Removed Reactions';
put storeRxns(store);
put /;
put 'Product Formation';
put sum(j, outer_obj(j)*storeV(j,store)):8:4;
put /;
put 'Growth Rate';
put sum(j, inner_obj(j)*storeV(j,store)):8:4;
put /;
put "Removed Reactions";
loop(j$(storeA(j,store)=0), put j.tl; );
put /; put/;

*IF YOU WANT TO PRINT OUT THE NON-ZERO FLUXES REMOVE THE * FROM THE FOUR LINES BELOW
*put "Non-Zero Fluxes at Optimal Biomass Production"
*put /;
*loop(j$(storeV(j,store)<>0), put j.tl;  put storeV(j,store):8:4;   put /; );
*put /;

);
putclose result;